#  AreaEd - an editor for rom-style areas, especially for
#  Dawn of the Ages  http://dawn.netcologne.de
#
#  Copyright (C) 2001 Peter Herth <p.herth@uni-koeln.de>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.






from GDK import *
from string import *
import string
from math import *
import Loader
trace = None
(NORTH, EAST, SOUTH, WEST, UP, DOWN)= range(6)
TRUE = 1
FALSE = 0

opposite_dir = [2, 3, 0, 1, 5, 4]

wearloc = { "0"  : "Light      ",     "13" : "Waist      ",
            "1"  : "Finger     ",     "14" : "Left Wrist ",
            "2"  : "Finger     ",     "15" : "Right Wrist",
            "3"  : "Neck       ",     "16" : "Wield      ",
            "4"  : "Tattoo     ",     "17" : "Held       ",
            "5"  : "On Torso   ",     "18" : "Floating   ",
            "6"  : "Head       ",     "19" : "Floating   ",
            "7"  : "Legs       ",     "20" : "Floating   ",
            "8"  : "Feet       ",     "21" : "Finger     ",
            "9"  : "Hands      ",     "22" : "Finger     ",
            "10" : "Arms       ",     "23" : "Finger     ",
            "11" : "Shield     ",     "24" : "Finger     ",
            "12" : "About Body ",     "25" : "Sheath     ",
            "26" : "Quiver     ",     "27" : "Second head",
            "28" : "Hind legs  ",     "29" : "Finger     ",
            "30" : "Finger     ",     "31" : "Finger     ",
            "32" : "Finger     ",     "33" : "Finger     "
            }


# increment vnum by 1 if anyhow possible

flags_to_locs = { "B" : [1,2,21,22,23,24,29,30,31,32,33],#Finger
                  "C" : [3], #Neck
                  "D" : [5], #body
                  "E" : [6], #head 
                  "F" : [7,28], #legs
                  "G" : [8], #feet
                  "H" : [9], #hands
                  "I" : [10],#arms
                  "J" : [11],#shield
                  "K" : [12],#aboutbody
                  "L" : [13],#waist
                  "M" : [14,15],#wrist
                  "N" : [16],#shield
                  "O" : [17],#hold
                  "P" : [0],#light
                  "Q" : [18,19,20],#float
                  "S" : [25],#sheath               
                  "T" : [4],#tattoo
                  "U" : [26]  #quiver
                  }

def get_wearloc(item, resets):
    locs = []
    for flag in item.wearflags:
        if flags_to_locs.has_key(flag):
            locs  = locs + flags_to_locs[flag]
##            for loc in flags_to_locs[flag]:
##                locs.append(loc)
##            print "get_wearloc:", item.short,"flags:",item.wearflags,"locs:", locs
    for loc in locs:
        can_take = 1
        for r in resets:
            if loc == int(r.v[4]):
                can_take = 0
        if can_take:
            return loc
    return -1

def vnum_inc(vnum):
    #    print "vnum-inc(",vnum,"):"
    l = len(vnum)
    if (l<1):
        return "1"
    else:
        if (vnum[-1]=="9"):
            return vnum_inc(vnum[:-1])+"0"
        elif (vnum[-1]=="Z"):
            return vnum_inc(vnum[:-1])+"A"
        else:
            return vnum[:-1]+chr(ord(vnum[-1])+1)
        
     
def room_cmp_by_z(r1,r2):
        if (r1.z<r2.z):
            return -1
        elif(r1.z>r2.z):
            return 1
        else:
            return 0         

def write_delim(file, str):
    if find(str, " ")>=0 or find(str, "\t")>=0 or len(str)<1:
        file.write("\"%s\"" % str)
    else:
        file.write(str)
    
class AreaObject:
    def __init__(self, vnum="1", name=""):
        self.vnum = vnum 
        self.name = name

    def save(self, file):
        pass


class Help:
    def __init__(self, name="", level=1, content=""):
        self.name = name
        self.level = level
        self.content = content
        print "Help: topic=%s level=%s\ntext=%s" %(name,level,content)
        self.subtopics = []

    def save(self, file):
        if (self.level<0):
            file.write('<topic name="%s">\n' % self.name)
        else:
            file.write('<topic name="%s" level="%s">\n' % (self.name, self.level))
        file.write(self.content)
        for topic in self.subtopics:
            topic.save(file)
        file.write("</topic>\n")
    
class Mobile(AreaObject):
    def __init__(self, vnum ="1"):
        self.vnum = vnum
        self.name = "nobody"
        self.short = "a nobody"
        self.long = "A nobody stands here looking undescript."
        self.look = "You look at a nobody"
        self.race = "human"
        self.align = 0
        self.mobgroup = 0
        self.level = 1
        self.toHit = "0"
        self.hpDice = "0d1"
        self.manaDice = "0d1"
        self.dtype="bash"
        self.damageDice="0d1"
        self.pierce = 0
        self.bash = 0
        self.slash = 0
        self.magic = 0
        self.start = "stand"
        self.default= "stand"
        self.sex = "male"
        self.treasure = 0
        self.size = "normal"
        self.material=""
	self.clan =""

        self.formflags = "0"
        self.partflags = "0"
        self.trigger = []
        self.scripts = []
        self.death = ""

        self.actflags = "0"
        self.offflags = "0"
        self.affflags = "0"
        self.iflags = "0"
        self.rflags = "0"
        self.vflags = "0"
        self.node = None
        
    def save(self, file):
        file.write("#%s\n%s~\n%s~\n" % (self.vnum, self.name, self.short))
        file.write(self.long)
        file.write("~\n")
        file.write(self.look)
        file.write("~\n")
        file.write(self.race)
        file.write("~\n")
        file.write(self.actflags)
        file.write(" ")
        file.write(self.affflags)
        file.write(" ")
        file.write("%d" % self.align)
        file.write(" ")
        file.write("%d" % self.mobgroup)
        file.write("\n")
        file.write("%d" % self.level)
        file.write(" ")
        file.write(self.toHit)
        file.write(" ")
        file.write(self.hpDice)
        file.write(" ")
        file.write(self.manaDice)
        file.write(" ")
        file.write(self.damageDice)
        file.write(" ")
        file.write(self.dtype)
        file.write("\n")
        file.write("%d" % self.pierce)
        file.write(" ")
        file.write("%d" % self.bash)
        file.write(" ")
        file.write("%d" % self.slash)
        file.write(" ")
        file.write("%d" % self.magic)
        file.write("\n")
        file.write(self.offflags)
        file.write(" ")
        file.write(self.iflags)
        file.write(" ")
        file.write(self.rflags)
        file.write(" ")
        file.write(self.vflags)
        file.write("\n")
        file.write(self.start)
        file.write(" ")
        file.write(self.default)
        file.write(" ")
        file.write(self.sex)
        file.write(" ")
        file.write("%d\n" % self.treasure)
        file.write(self.formflags)
        file.write(" ")
        file.write(self.partflags)
        file.write(" ")
        file.write(self.size)
        file.write(" ")
        file.write(self.material)
        file.write("\n")
	if (len(self.clan)>0):
	    file.write("C %s\n" % self.clan)
        if (len(self.death)>0):
            file.write("D ")
            file.write(self.death)
            file.write("~\n")
        for t in self.trigger:
            file.write("%s %s %s %s~\n" % t)
        #file.write("\n")
        
class Item(AreaObject):
    def __init__(self, vnum="1"):
        self.vnum = vnum
        self.name = ""
        self.short= ""
        self.long = ""
        self.look = ""
        self.material = ""
        self.type = "trash"
        self.extraflags =""
        self.wearflags = ""
        self.v0="0"
        self.v1="0"
        self.v2="0"
        self.v3="0"
        self.v4="0"
        self.level=1
        self.weight=0
        self.cost=0
        self.timer="P"
        self.extras=[]
        
        self.crumble = ""
        self.useentries = []
        self.trigger = []
        self.applies = []
        self.resistancies = []
        self.raceapplies = []
        self.flags = []
        self.node = None
        
    def save(self, file):
        file.write("#%s\n%s~\n%s~\n" % (self.vnum, self.name, self.short))
        file.write(self.long)
        file.write("~\n")
        file.write(self.material)
        file.write("~\n")
        file.write(self.type)
        file.write(" ")
        file.write(self.extraflags)
        file.write(" ")
        file.write(self.wearflags)
        file.write("\n")
        write_delim(file, self.v0)
        file.write(" ")
        write_delim(file, self.v1)
        file.write(" ")
        write_delim(file, self.v2)
        file.write(" ")
        write_delim(file, self.v3)
        file.write(" ")
        write_delim(file, self.v4)
        file.write("\n")
        file.write("%d" % self.level)
        file.write(" ")
        file.write("%d" % self.weight)
        file.write(" ")
        file.write("%d" % self.cost)
        file.write(" ")
        file.write(self.timer)
        file.write("\n")
        if (len(self.look)>0):
            file.write("L\n")
            file.write(self.look)
            file.write("~\n")
            
        if (len(self.crumble)>0):
            file.write("C ")
            file.write(self.crumble)
            file.write("\n")
        for u in self.useentries:
            file.write("U %s %s\n" % u)
        for t in self.trigger:
            file.write("T %s %s %s~\n" % t)
        for a in self.applies:
            file.write("A %d %d\n" % a)
        for i in self.resistancies:
            file.write("I %d %d\n" % i)
        for r in self.raceapplies:
            file.write("R %s %d %d\n" % r)
        for f in self.flags:
            file.write("F %s %d %d %s\n" % f)
        for e in self.extras:
            file.write("E\n%s~\n%s~\n" % (e.name, e.text))

        #file.write("\n")


class Mobprog:
    def __init__(self, name, code):
        self.name = name
        self.code = code

class Extra:
    def __init__(self, name, text):
        self.name = name
        self.text = text

class Script:
    def __init__(self, name, code):
        self.name = name
        self.code = code
        
class Door:
    def __init__(self, dir):
        self.dir = dir
        self.lookdesc = ""
        self.keywords = ""
        self.state = "0"
        self.key = 0
        self.roomTo = 0

    def save(self, file):
        file.write("D%d\n" % self.dir)
        file.write(self.lookdesc)
        file.write("~\n")
        file.write(self.keywords)
        file.write("~\n")
        file.write(self.state)
        file.write(" ")
        file.write("%d " % self.key)
        file.write("%d\n" % self.roomTo)
        
        
class Exit:
    def __init__(self):
        self.dir = -1
        self.desc = ""
        self.keyphrase = ""
        self.doorstate = "0"
        self.keyVnum = "-1"
        self.roomToVnum = "0"
        self.line = None
        
    def save(self, file):
        file.write("D%d\n" % self.dir)
        file.write(self.desc)
        file.write("~\n")
        file.write(self.keyphrase)
        file.write("~\n")
        file.write(self.doorstate)
        file.write(" ")
        file.write(self.keyVnum)
        file.write(" ")
        file.write(self.roomToVnum)
        file.write("\n")

    def render(self, r, canvas, area, recurse=None):
        root = canvas.root()
        dir = self.dir
        to = None
        
        if area.r_v.has_key(self.roomToVnum):
            to = area.r_v[self.roomToVnum]
            if dir == 0:
                p = [r.cx+1, r.y1, to.cx+1, to.y2]
            elif dir == 1:
                p = [r.x2, r.cy+1, to.x1, to.cy+1]
            elif dir == 2:
                p = [r.cx-1, r.y2, to.cx-1, to.y1]
            elif dir == 3:
                p = [r.x1, r.cy-1, to.x2, to.cy-1]
            elif dir == 4:
                p = [r.cx, r.cy, r.cx+10, r.cy-10]
            elif dir == 5:
                p = [r.cx, r.cy, r.cx-10, r.cy+10]
        else:
            to = None
            if dir == 0:
                p = [r.cx+1, r.y1, r.cx+1, r.y1-10]
            elif dir == 1:
                p = [r.x2, r.cy+1, r.x2+10, r.cy+1]
            elif dir == 2:
                p = [r.cx-1, r.y2, r.cx-1, r.y2-10]
            elif dir == 3:
                p = [r.x1, r.cy-1, r.x1-10, r.cy-1]
            elif dir == 4:
                p = [r.cx, r.cy, r.cx+10, r.cy-10]
            elif dir == 5:
                p = [r.cx, r.cy, r.cx-10, r.cy+10]
        


        if self.line:
            self.line.set(points = p)
            if self.doorstate == "0":
                self.line.set(line_style=LINE_SOLID)
            else:
                self.line.set(line_style=LINE_ON_OFF_DASH) 

        else:
            if self.doorstate == "0":
                self.line = root.add("line", points = p,
                                     width_units=1,cap_style=CAP_PROJECTING,
                                     arrow_shape_a=2, arrow_shape_b=3, arrow_shape_c=1.5,
                                     first_arrowhead=0,last_arrowhead=1,line_style=LINE_SOLID)
            else:
                self.line = root.add("line", points = p,
                                     width_units=1,cap_style=CAP_PROJECTING,
                                     arrow_shape_a=2, arrow_shape_b=3, arrow_shape_c=1.5,
                                     first_arrowhead=0,last_arrowhead=1,line_style=LINE_ON_OFF_DASH)
            
                
           

        opdir = opposite_dir[dir]
        
        if recurse and to.exits[opdir]:
            to.exits[opdir].render(to, canvas, area)


        
    
class Room(AreaObject):
    def __init__(self, area, vnum ="1"):
        self.area = area
        self.roomflags =""
        self.vnum=vnum
        self.name=""
        self.desc=""
        self.flags="0"
        self.sector="1"
        self.doors=[]
        self.x=0
        self.y=0
        self.z=0
        self.width = 10
        self.length = 10
        self.height = 10
        self.exits = []
        self.extras = []
        self.rect = None
        self.x1 = 0
        self.y1 = 0
        self.x2 = 10
        self.y2 = 10
        self.cz = 5
        self.cx = 5
        self.cy = 0
        self.drag = 0
        self.heal = 100
        self.mana = 100
        self.node = None
        self.exits = [None, None, None, None, None, None][:]
        self.hasCoords = 0
        self.resets = []
        self.canvas = None

    def add_reset(self, reset):
        self.resets.append(reset)
        
    def calcCoords(self):
        self.x1 = self.x + self.z/4 - self.width/2
        self.y1 = self.y - self.z/4 - self.length/2
        self.x2 = self.x1 + self.width
        self.y2 = self.y1 + self.length
        self.cx = (self.x1 + self.x2)/2
        self.cy = (self.y1 + self.y2)/2
        if self.rect:
            self.rect.set(x1=self.x1, y1=self.y1, x2=self.x2, y2=self.y2)
        
        
    def save(self, file):
        file.write("#%s\n%s~\n%s~\n" % (self.vnum, self.name, self.desc))
        file.write("0 ")
        file.write(self.flags)
        file.write(" ")
        file.write(self.sector)
        file.write("\n")
        for e in self.exits:
            if e:
                e.save(file)
        for e in self.extras:
            file.write("E\n%s~\n%s~\n" % (e.name, e.text))
        if self.hasCoords:
            file.write("K xyz %d %d %d\n" % (self.x, self.y, self.z))
            if (self.width != 10 or self.height != 10 or self.length != 10):
                file.write("WLH %d %d %d\n" % (self.width, self.length, self.height))
        if (self.heal != 100):
            file.write("H %d " % self.heal)
        if (self.mana != 100):
            file.write("M %d" % self.mana)
        if (self.heal != 100) or (self.mana != 100):
            file.write("\n")
        file.write("S\n")

    def render(self, canvas, roomform):
        self.canvas = canvas
        self.form = roomform
        root = canvas.root()
        if self.rect:
            self.rect.set(x1=self.x1, y1=self.y1, x2=self.x2, y2=self.y2)
        else:
            rect = root.add("rect", x1=self.x1, y1=self.y1, x2=self.x2, y2=self.y2,
                            fill_color="lightgray", outline_color="black", width_pixels=2)
            self.rect = rect
            rect.connect("event", self.room_event)
        canvas.update_now()

    def room_event(self, *args):
        e = args[1]
        if (e.type == BUTTON_PRESS):
            self.drag = 1
            self.form.set(self)
        elif (e.type == BUTTON_RELEASE):
            self.drag = 0
        elif (e.type == MOTION_NOTIFY):
            if (self.drag):
                x = round(e.x/10)
                y = round(e.y/10)
                x = x * 10
                y = y * 10
                #room.rect.move(x,y)
                self.x = x
                self.y = y
##                self.x1 = x
##                self.y1 = y
##                self.x2 = x+10
##                self.y2 = y+10
##                self.cx = x+5
##                self.cy = y+5
##                self.rect.set(x1=self.x1, y1=self.y1, x2= self.x2, y2= self.y2)
                self.calcCoords()
                self.form.koordEd.setKoords(self)
                self.render(self.canvas, self.form)
                self.redrawExits()
                self.canvas.update_now()

    def redrawExits(self):
        if self.canvas!=None:
            for r in self.area.rooms:
                for e in r.exits:
                    if e:
                        if e.roomToVnum==self.vnum:
                            e.render(r,self.canvas,self.area,FALSE)

            for e in self.exits:
                if e:
                    e.render(self, self.canvas, self.area, FALSE)

            self.canvas.update_now()
                
    def a(self, d,sign):
        d = d*sign
        if d<20:
            return sign * max(-5,(d-20))
        elif d==0:
            return 0
        else:
            return sign * (d-20)/40
        

class Reset:
    def __init__(self):
        self.v = []
        self.resets = []
        self.mob = None
        self.item = None
        self.super = None
        self.short = None

    def add_reset(self, reset):
        self.resets.append(reset)

    def __repr__(self):
        return string.join(self.v," ")

    def save(self, file):
        file.write("%s\n" % self)
        for r in self.resets:
            r.save(file)

class Shop:
    def __init__(self):
        self.itemtypes = []

class Special:
    pass

class Area:
    def __init__(self):
        self.mobs = []
        self.helps = []
        self.rooms = []
        self.items = []
        self.resets = []
        self.mobprogs = []
        self.scripts = []
        self.shops = []
        self.specials = []
        
        self.filename = "x"
        self.areaname = "x"
        self.command="x"
        self.vnums ="x x"
        self.r_v = {}
        self.m_v = {}
        self.resets_sorted = None

    def load(self,t):
        w = t.readWord()
        while w:
            print "w:",w
            if w=="#AREA":
                self.readHeader(t)
            elif w=="#HELPS":
                self.read_oldhelps(t)
            elif w=="<help>":
                self.read_helps(t)
            elif w== "#ROOMS":
                self.readRooms(t)
            elif w=="#MOBILES":
                self.readMobs(t)
            elif w=="#OBJECTS":
                self.readItems(t)
            elif w=="#RESETS":
                self.readResets(t)
            elif w=="#MOBPROGS":
                self.readMobprogs(t)
            elif w=="#SCRIPTS":
                self.readScripts(t)
            elif w=="#SHOPS":
                self.readShops(t)
            elif w=="#SPECIALS":
                self.readSpecials(t)

            w = t.readWord()
            #progress.set_percent(t.readpos/n)
            #        print "Found rooms!"
        self.sort_resets()

    def save(self, filename):
        file = open(filename, "w")
        file.write("#AREA\n")
        file.write(self.filename)
        file.write("~\n")
        file.write(self.areaname)
        file.write("~\n")
        file.write(self.command)
        file.write("~\n")
        file.write(self.vnums)
        file.write("\n\n")

        if len(self.helps)>0:
            file.write("<help>\n")
            for topic in self.helps:
                topic.save(file)
            file.write("</help>\n\n")
        
        if len(self.mobs)>0:
            file.write("#MOBILES\n")
            for mob in self.mobs:
                mob.save(file)
            file.write("#0\n\n")

        if len(self.items)>0:
            file.write("#OBJECTS\n")
            for item in self.items:
                item.save(file)
            file.write("#0\n\n")

        if len(self.rooms)>0:
            file.write("#ROOMS\n")
            for room in self.rooms:
                room.save(file)
            file.write("#0\n\n")

        if len(self.shops)>0:
            self.writeShops(file)

        if len(self.specials)>0:
            self.writeSpecials(file)

        if len(self.resets)>0:
            file.write("#RESETS\n")
            for room in self.rooms:
                for reset in room.resets:
                    reset.save(file)
                    
##            for r in self.resets:
##                r.save(file)  #file.write(r)
##                file.write("\n")
            file.write("S\n")

        self.writeScripts(file)
        self.writeMobprogs(file)
            

        file.write("\n#$\n")
        
        file.close()



    def readHeader(self, ft):
        self.filename = ft.readLong()
        self.areaname = ft.readLong()
        self.command = ft.readLong()
        self.vnums = ft.readWord()+" "+ft.readWord()
        
    def read_oldhelps(self, ft):
        line = ft.readLong()
        while (line and not line[:3]=="0 $"):
            t = Loader.Tokenizer(line)
            level = t.readInt()
            keys = string.lstrip(t.readRest())
            text = ft.readLong()
            self.helps.append(Help(keys, level, text))
            line = ft.readLong()

    def read_helps(self, ft):
        ft.readLine()
        print "load new helps"

        line = ft.readLine()
        print "read_help:",line
        while not (line[:6]=="</help"):
            if (string.find(line,"<topic")>=0):
                print "found topic"
                self.helps.append(self.readTopic(ft,line))
            else:
                print "no topic:", line
            line = ft.readLine()

    def readTopic(self, ft, line):
        print "readTopic:",line
        #line = string.lower(line)
        pos = string.find(line,"reference=")
        if (pos>0):
            print "referencetopic!:",line
            return Help(line,1,line)

        pos = string.find(line,"name=")
        if pos<0:
            return None
        level = -1
        while line[pos]!='"':
            pos = pos + 1
        pos = pos + 1
        cut_start = pos
        while line[pos]!='"':
            pos = pos + 1
        name = line[cut_start:pos]
        print "topicname:",name

        line = line[pos:]
        
        pos = string.find(line,"level=")
        if pos>0:
           while line[pos]!='"':
               pos = pos + 1
           pos = pos + 1
           cut_pos = pos
           while line[pos]!='"':
               pos = pos + 1
           level = int(line[cut_pos:pos])

        buffer = ""
        topic = Help(name, level, buffer)
        
        line = ft.readLine()
        while string.find(line,"</topic")<0:
            if string.find(line,"<topic")>=0:
                topic.subtopics.append(self.readTopic(ft, line))
            else:
                topic.content = topic.content + line + "\n"
            line = ft.readLine()

        return topic


    def readMobs(self,ft):
        w=ft.readWord();
        while w and w!="#0":
            if w[0]=="#":
                #progress.set_percent(ft.readpos/ft.max)
                m = Mobile()
                self.mobs.append(m)
                m.vnum = w[1:]
                self.m_v[m.vnum] = m
                #print "vnum:",m.vnum
                m.name = ft.readLong()
                m.short = ft.readLong()
                m.long = ft.readLong()
                m.look = ft.readLong()
                m.race = ft.readLong()
                m.actflags = ft.readWord()
                m.affflags = ft.readWord()
                m.align = ft.readInt()
                m.mobgroup = ft.readInt()
                m.level = ft.readInt()
                m.toHit = ft.readWord()
                m.hpDice = ft.readWord()
                m.manaDice = ft.readWord() #       m.hitDice    = new Dice(ft.readWord());
                m.damageDice = ft.readWord()
                m.dtype = ft.readWord()
                m.pierce = ft.readInt()
                m.bash = ft.readInt()
                m.slash = ft.readInt()
                m.magic = ft.readInt()
                m.offflags = ft.readWord()
                m.iflags = ft.readWord()
                m.rflags = ft.readWord()
                m.vflags = ft.readWord()
                m.start = ft.readWord()
                m.default = ft.readWord()
                m.sex = ft.readWord()
                m.treasure = ft.readInt()
                m.formflags = ft.readWord()
                m.partflags = ft.readWord()
                m.size = ft.readWord()
                m.material = ft.readWord()
                w = ft.readWord()
                while w[0] != "#":
                    if w=="H":
                        m.hometown = ft.readWord
                    elif w == "E":
                        m.ethos = ft.readWord
                    elif w == "M" or w=="T":
                        key = ft.readWord()
                        prog = ft.readWord()
                        arg = ft.readLong()
                        m.trigger.append((w,key,prog,arg))
                        #add trigger
                    elif w=="S":
                        sname = ft.readWord()
                        speed = ft.readInt()
                    elif w=="C":
                        m.clan = ft.readWord()
                    elif w=="D":
                        m.death = ft.readLong()
                    elif w=="App":
                        m.app = ft.readInt()
                    elif w=="Str":
                        m.str = ft.readInt()
                    elif w=="Dex":
                        m.dex = ft.readInt()
                    elif w=="Int":
                        m.int = ft.readInt()
                    elif w=="Wis":
                        m.wis = ft.readInt()
                    elif w=="Qui":
                        m.qui = ft.readInt()
                    elif w=="Cha":
                        m.cha = ft.readInt()
                    elif w=="Con":
                        m.con = ft.readInt()
                    w = ft.readWord()
            while w and w[0]!="#":
                w = ft.readWord()

    def readRooms(self,ft):
        w = ft.readWord()
        r = None
        while ( w and not w=="#0"):
            if (w[0]=="#"):
                r=Room(self);
                #progress.set_percent(ft.readpos/ft.max)

                #self.addRoom(r);
                self.rooms.append(r)
                r.vnum = w[1:]
                self.r_v[r.vnum]=r
                #print "Room(%d)" % r.vnum
                r.name = ft.readLong()
                r.desc	= ft.readLong()

                ft.readWord();
                r.flags = ft.readWord() #ft.readFlags();
                
                r.sector    =  ft.readWord() #ft.readInt();                
                #       r.sector = RoomSector.getSector(sector);
                #       if (r.sector == RoomSector.UNKNOWN)
                #	{
                #	 world.bug("Unknown room sector in line "+ft.getLine());
                #	}
                w = ft.readWord()
                while not w =="S":
                    if ( w[0]=='D' ):
                        #print "Door:",w
                        #	   if (r.nexit>5)
                        #	     world.log("Buggy door:"+w+"("+r.nexit+")"+" in line "+ft.getLine());
                        #	   jmud.room.Exit e = new jmud.room.Exit();
                        #	   r.exit[r.nexit++] = e;
                        #	   if (w.length()<2)
                        #	     world.log("Buggy door:"+w+" in line "+ft.getLine());
                        #	   else
                        #	     e.dir         =  (byte) (w.charAt(1)-'0');
                        e = Exit()
                        e.dir = int(w[1:])
                        e.desc        = ft.readLong()
                        e.keyphrase   = ft.readLong()
                        e.doorstate  = ft.readWord()
                        e.keyVnum    = ft.readWord() #Vnum.read(ft.readWord(),nr);//ft.readInt();
                        e.roomToVnum = ft.readWord() #Vnum.read(ft.readWord(),nr);//ft.readInt();
                        r.exits[e.dir] = e
                        #r.exits.append(e)
                    elif w=="E":
                        key = ft.readLong()
                        text = ft.readLong()
                        r.extras.append(Extra(key,text))
                        #             r.addExtra(ft.readLong(), color(ft.readLong()));
                    elif w=="K":
                        w = ft.readWord()
                        r.hasCoords = 1
                        r.x = ft.readInt() # # x
                        r.y = ft.readInt() # # y
                        r.z = ft.readInt() # # z
                    elif w=="WLH":
                        r.width = ft.readInt()
                        r.length = ft.readInt()
                        r.height = ft.readInt()
                    elif w=="H":         
                        r.heal = ft.readInt() # hp  
                    elif w=="M":         
                        r.mana = ft.readInt() # mana
                    elif  w=="X":
                        w = ft.readWord() # spell                
                        w = ft.readWord() # v0
                        w = ft.readWord()
                        w = ft.readWord()
                        w = ft.readWord()
                        w = ft.readWord() # v4
                    elif w=="clan":
                        w = ft.readWord()
                        w = ft.readWord()
                    w = ft.readWord()
            while w and not w[0]=="#":
                w = ft.readWord()
            r.calcCoords()
            
    def readItems(self, ft):
        w=ft.readWord()

        while w and w!="#0":
            if w[0]=="#":
                i = Item()
                self.items.append(i)
                i.vnum = w[1:]
                i.name = ft.readLong()
                if trace:
                    print "Item: %s" % i.vnum
                i.short = ft.readLong()
                i.long = ft.readLong()
                i.material = ft.readLong()
                i.type = ft.readWord()
                i.extraflags = ft.readWord()
                i.wearflags = ft.readWord()
                i.v0 = ft.readDelim()
                i.v1 = ft.readDelim()
                i.v2 = ft.readDelim()
                i.v3 = ft.readDelim()
                i.v4 = ft.readDelim()
            
                i.level = ft.readInt()
                i.weight = ft.readInt()
                i.cost = ft.readInt()
                i.timer = ft.readWord()

                w = ft.readWord()

                while w[0] != "#":
                    if w=="L":
                        i.look = ft.readLong()
                    elif w=="C":
                        i.crumbleMessage = ft.readLong()
                    elif w=="U":
                        tool = ft.readDelim()
                        prog = ft.readWord()
                        i.useentries.append((tool, prog))
                    elif w=="T":
                        key = ft.readWord()
                        prog= ft.readWord()
                        arg = ft.readLong()
                        i.trigger.append((key,prog,arg))
                    elif w=="A":
                        location = ft.readInt()
                        modifier = ft.readInt()
                        i.applies.append((location, modifier))
                    elif w == "I":
#                        location = ft.readInt()
                        damtype  = ft.readInt()
                        modifier = ft.readInt()
                        i.resistancies.append((damtype, modifier))
                    elif w == "R":
                        races    = ft.readWord()
                        location = ft.readInt()
                        modifier = ft.readInt()
                        i.raceapplies.append((races,location,modifier))
                    elif w == "F":
                        airf     = ft.readWord()
                        location = ft.readInt()
                        modifier = ft.readInt()
                        bits     = ft.readWord()
                        i.flags.append((airf,location,modifier,bits))
                    elif w == "E":
                        key = ft.readLong()
                        text = ft.readLong()
                        i.extras.append(Extra(key,text))
                    w = ft.readWord()
            while w and not w[0]=="#":
                w = ft.readWord()
       
    def readResets(self,ft):
        l = lstrip(ft.readLine())
        while  not (len(l)>0 and l[0] == "S"):
            if len(l) > 0:
                if l[0] != "*":
                    r = Reset()
                    r.v = split(l)
                    self.resets.append(r)
            l = lstrip(ft.readLine())

    def readMobprogs(self,ft):
        w = ft.readWord()
        while w and w!="#0":
            if w[0]=="#":
                name = w[1:]
                code = ft.readLong()
                self.mobprogs.append(Mobprog(name,code))
            w = ft.readWord()

    def writeMobprogs(self,file):
        if len(self.mobprogs)>0:
            file.write("#MOBPROGS\n")
            for p in self.mobprogs:
                file.write("#%s\n%s~\n" % (p.name, p.code))
            file.write("#0\n")

    def writeScripts(self,file):
        if len(self.scripts)>0:
            file.write("#SCRIPTS\n")
            for s in self.scripts:
                file.write("#%s\n%s~\n\n\n" % (s.name, s.code))
            file.write("#0\n\n")
        
    def readScripts(self,ft):
        w = ft.readWord()
        while w and w!="#0":
            if w[0]=="#":
                name = w[1:]
                code = ft.readLong()
                print "read script:",name," code:",code
                self.scripts.append(Script(name,code))
            w = ft.readWord()




    def readShops(self, ft):
        w = ft.readWord()
        while w and w!= "0":
            s = Shop()
            self.shops.append(s)
            s.keeper = w
            for i in range(5):
                s.itemtypes.append(ft.readWord())
            s.percentBuy  = ft.readWord()
            s.percentSell = ft.readWord()
            s.openHour	  = ft.readWord()
            s.closeHour	  = ft.readWord()
            s.maxNumber   = ft.readWord()
            s.shopSpeed	  = ft.readWord()
            s.initNumber  = ft.readWord()
    
            s.comment = ft.readLine()
            w = ft.readWord()


    def writeShops(self, file):
        file.write("#SHOPS\n")
        for s in self.shops:
            file.write("%s %s %s %s %s %s %s %s %s %s %s %s %s\n" %
                       (s.keeper, s.itemtypes[0], s.itemtypes[1],s.itemtypes[2],
                        s.itemtypes[3], s.itemtypes[4], s.percentBuy, s.percentSell,
                        s.openHour, s.closeHour, s.maxNumber, s.shopSpeed, s.initNumber))
        file.write("0\n")

    def readSpecials(self, ft):
        w = ft.readWord()
        while w and w!="S":
            s = Special()
            self.specials.append(s)
            if w !="M":
                print "strange special:", w
            s.vnum = ft.readWord()
            s.prog = ft.readWord()
            s.comment = ft.readLine()
            w = ft.readWord()

    def writeSpecials(self, file):
        file.write("#SPECIALS\n")
        for s in self.specials:
            file.write("M %s %s %s\n" % (s.vnum, s.prog, s.comment))
        file.write("S\n")
     

    def get_room(self,vnum):
        for r in self.rooms:
            if (r.vnum == vnum):
                return r
        return None

    def get_mob(self,vnum):
        for m in self.mobs:
            if (m.vnum == vnum):
                return m
        return None

    def get_item(self,vnum):
        for m in self.items:
            if (m.vnum == vnum):
                return m
        return None
    
    
    def sort_resets(self):
        if self.resets_sorted:
            return
        self.resets_sorted = TRUE
        print "sorting..."
        for r in self.resets:
            if r.v[0]=="M":
                room = self.get_room(r.v[4])
                if room:
                    room.add_reset(r)
                    last_mob = r
                else:
                    print "Mobile reset into other area:",r.v[2],"into room", r.v[4]
            elif r.v[0]=="E":
                if last_mob:
                    last_mob.add_reset(r)
                else:
                    print "Cannot find mobile for equip:%s" % r
            elif r.v[0]=="G":
                if last_mob:
                    last_mob.add_reset(r)
                else:
                    print "Cannot find mobile for give:%s" % r
            elif r.v[0]=="O":
                room = self.get_room(r.v[4])
                if room:
                    room.add_reset(r)
                else:
                    print "Item reset into other area:",r.v[2],"into room", r.v[4]
            elif r.v[0]=="D":
                room = self.get_room(r.v[2])
                if room:
                    room.add_reset(r)
                else:
                    print "Door reset into other area:",r.v[2]
            elif r.v[0]=="R":
                room = self.get_room(r.v[2])
                if room:
                    room.add_reset(r)
                else:
                    print "Random reset into other area:",r.v[2]
            elif r.v[0]=="P":
                cvnum = r.v[4]
                found = None
                for re in self.resets:
                    if re.v[0]=="O" and re.v[2]==cvnum:
                        re.add_reset(r)
                        found = 1
                        break
                if not found:
                    print "Cannot find an object in the area to put in ",r.v[2]


    def automap(self):
        x = y = z = 0;

        maxx = -9999999;
        minx = 99999999;
        maxy = -9999999;
        miny = 99999999;

        for r in self.rooms:
            if not r.hasCoords:
                r.x = x
                r.y = y
                r.z = z
                
                if (x>maxx): maxx=x
                if (x<minx): minx=x
                if (y>maxy): maxy=y
                if (y<miny): miny=y
                r.hasCoords = 1
                r.calcCoords()

                self.map_room(r)
                y = y + 20
            else:
                if (r.x>maxx):  maxx =r.x
                if (r.x<minx):  minx =r.x
                if (r.y>maxy):  maxy =r.y
                if (r.y<miny):  miny =r.y  

    def map_room(self, room):
        ##print "mapping room ",room.vnum,room.name
        
        x=0
        y=0
        z=0
        torender = []

        for e in room.exits:
            if e:
                to = self.get_room(e.roomToVnum)
                if to and (not to.hasCoords):
                    torender.append(to)
                    to.x = room.x
                    to.y = room.y
                    to.z = room.z
                    if e.dir == 0:
                        to.y = to.y - 20
                    elif e.dir == 1:
                        to.x = to.x + 20
                    elif e.dir == 2:
                        to.y = to.y + 20
                    elif e.dir == 3:
                        to.x = to.x - 20
                    elif e.dir == 4:
                        to.z = to.z + 20
                    elif e.dir == 5:
                        to.z = to.z - 20
                    to.hasCoords = 1
                    to.calcCoords()

        for r in torender:
            self.map_room(r)

        

    def find_room(self, x,y,z):
        for r in self.rooms:
            if r.x == x and r.y == y and r.z == z:
                return r
        return None

    def find_roomnr(self, room):
        for i in range(len(self.rooms)):
            if self.rooms[i]==room:
                return i
        return None
