#  AreaEd - an editor for rom-style areas, especially for
#  Dawn of the Ages  http://dawn.netcologne.de
#
#  Copyright (C) 2001 Peter Herth <p.herth@uni-koeln.de>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.



from gtk import *
from gnome.ui import *
from GtkExtra import *
from GDK import *
from GTK import *

from Tools import *
import Area
#from Area import *


class MobForm(EditForm):
    actflags = []
    offflags = []
    affflags = []
    irvflags = []
    partflags = []
    formflags = []
    
    def __init__(self, ed):
        EditForm.__init__(self, ed, ed.area.mobs)
        self.ed = ed
        self.tooltips = GtkTooltips()
        mklabel("look desc:",self,0,5)
        self.vnum = mk_le("vnum:",self,0,1)

        self.name = mk_le("Name:", self, 0,2,8)
        self.short = mk_le("Short:", self, 0,3,8)
        self.long = mk_le("Long:", self, 0,4,8)
        
        look = GtkText()
        look.set_editable(TRUE)
        look.set_usize(1,MOBDESCHEIGHT)
        look.show()
        self.attach(look,1,8,5,7,FILL|EXPAND|SHRINK,EXPAND|FILL, XPAD, YPAD)
        self.look = look

        self.level = mk_le("level:",self,0,7)
        self.race = mk_cb("race:",self,2,7)
        file = open("races.dat", "r")
        races = []
        for line in file.readlines():
            races.append(string.strip(line))
        file.close()
        self.race.set_popdown_strings(races)

        
        self.align = mk_le("align:",self, 4,7)
        
        self.mobgroup = mk_le("group:",self,6,7)
        
        self.dtype = mk_le("dtype:",self,0,8)
        self.toHit = mk_le("+hit:",self,2,8)
        self.hpDice = mk_le("hit:",self,4,8)
        #mana = mk_le("mana",self,6,8)
        self.damageDice = mk_le("damage:",self, 6,8)
        
        self.pierce = mk_le("pierce:",self,0,9)
        self.bash = mk_le("bash:",self,2,9)
        self.slash = mk_le("slash:",self,4,9)
        self.magic = mk_le("magic:",self,6,9)
        
        self.start = mk_le("start:",self,0,10)
        self.default = mk_le("default:",self,2,10)
        self.sex=mk_le("sex:",self,4,10)
        self.treasure=mk_le("treasure",self,6,10)

        self.death = mk_le("deathmsg:",self, 0, 11, 9)

        note_flags = GtkNotebook()
        note_flags.set_tab_pos = POS_TOP

        f = GtkFrame("Flags")
        note_flags.append_page(createFlags("actflag.dat",self.actflags, self.tooltips), GtkLabel("Action"))
        note_flags.append_page(createFlags("affectflag.dat", self.affflags, self.tooltips), GtkLabel("Affect"))
        note_flags.append_page(createFlags("offensiveflag.dat", self.offflags, self.tooltips), GtkLabel("Offensive"))

        note_flags.append_page(self.createIRV(self.irvflags), GtkLabel("I-R-V") )
        note_flags.append_page(createFlags("partflag.dat",self.partflags, self.tooltips), GtkLabel("Part"))
        note_flags.append_page(createFlags("formflag.dat",self.formflags, self.tooltips), GtkLabel("Form"))
        
        f.add(note_flags)
        self.attach(f, 1, 8, 12, 15,FILL,FILL)
        self.show()

        self.tooltips.set_tip(self.vnum, "The vnum of a mobile is a unique identifier")
        self.tooltips.set_tip(self.name,
                              "List of names with which the mobile can be "+
                              "referred by the player, should contain every noun "+
                              "in the short name")
        self.tooltips.set_tip(self.short,
                              "The name seen when interacting with the mobile, for "+
                              "example \"Hassan\" or \"a small frog\" ")
        self.tooltips.set_tip(self.long,
                              "The line a player gets when looking in the room, the mobile is in")
        self.tooltips.set_tip(self.look,
                              "The description of the mobile, when the player looks at it")
        self.tooltips.set_tip(self.level, "The level (1-60) of the mobile ")
        self.tooltips.set_tip(self.race.entry, "The race of the mobile")
        self.tooltips.set_tip(self.align, "The alignment of the mobile (1000 good - 0 neutral - -1000 evil)")
        self.tooltips.set_tip(self.mobgroup, "The number of the mobgroup the mobile should be in")
        self.tooltips.set_tip(self.dtype, "The damage type of the mobile")
        self.tooltips.set_tip(self.toHit, "The bonus on hitting")
        self.tooltips.set_tip(self.hpDice, "The hp of the mobile as a dice (xdy+z)")
        self.tooltips.set_tip(self.damageDice, "The damroll of the mobile as a dice (xdy+z)")
        self.tooltips.set_tip(self.bash, "The armor class against bash damage")
        self.tooltips.set_tip(self.pierce, "The armor class against pierce damage")
        self.tooltips.set_tip(self.slash, "The armor class against slash damage")
        self.tooltips.set_tip(self.magic, "The armor class against magic damage")
        self.tooltips.set_tip(self.death, "The message which is generated when the mobile dies "+
                              "(may contain any ACT control char), or empty for the standard message")
        self.tooltips.set_tip(self.start, "The position the mobile has after creation")
        self.tooltips.set_tip(self.default, "The default position of the mobile")
        self.tooltips.set_tip(self.sex, "The gender of the mobile")
        self.tooltips.set_tip(self.treasure, "The money of the mobile in silver coins")
        

    def createIRV(self, list):
        table = GtkTable(4,24)
        mklabel("Imm",table, 1,0)
        mklabel("Res",table, 2,0)
        mklabel("Vul",table, 3,0)

        file = open("irvflag.dat", "r")
        y = 1
        for line in file.readlines():
            l = string.split(line,"|")
            if len(l)>1:

                label = mklabel(l[1],table, 0, y)
                c1 = mk_chk("", table, 1, y)
                c2 = mk_chk("", table, 2, y)
                c3 = mk_chk("", table, 3, y)
                y = y + 1
                self.tooltips.set_tip(c1,l[2])
                self.tooltips.set_tip(c2,l[2])
                self.tooltips.set_tip(c3,l[2])            
                list.append( (l[0], c1, c2, c3))                    
        file.close()

        swin = GtkScrolledWindow()
        swin.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
        swin.add_with_viewport(table)
        return swin                 

    def setValues(self, mob):
        mob.vnum = self.vnum.get_text()
        mob.name = self.name.get_text()
        mob.short = self.short.get_text()
        mob.long = self.long.get_text()
        mob.look = self.look.get_chars(0,self.look.get_length())
        
        mob.race = self.race.entry.get_text()
        mob.align = int(self.align.get_text())
        mob.mobgroup = int(self.mobgroup.get_text())
        mob.level = int (self.level.get_text())
        mob.toHit = self.toHit.get_text()
        mob.hpDice = self.hpDice.get_text()
        mob.dtype = self.dtype.get_text()
        mob.damageDice = self.damageDice.get_text()
        mob.pierce = int(self.pierce.get_text())
        mob.bash = int(self.bash.get_text())
        mob.slash = int(self.slash.get_text())
        mob.magic = int(self.magic.get_text())
        mob.start = self.start.get_text()
        mob.default = self.default.get_text()
        mob.sex = self.sex.get_text()
        mob.treasure = int(self.treasure.get_text())
#        mob.size = self.size.get_text()
        mob.death = self.death.get_text()

        #self.triggers = []
        #self.scripts = []
        #self.death = ""

        mob.actflags = getFlags(self.actflags)
        mob.offflags = getFlags(self.offflags)
        mob.affflags = getFlags(self.affflags)
        mob.partflags = getFlags(self.partflags)
        mob.formflags = getFlags(self.formflags)

        mob.iflags = ""
        mob.rflags = ""
        mob.vflags = ""

        for f in self.irvflags:
            (flag, i, r, v) = f
            if (i.get_active()):
                mob.iflags = mob.iflags + flag
            if (r.get_active()):
                mob.rflags = mob.rflags + flag
            if (v.get_active()):
                mob.vflags = mob.vflags + flag

        if len(mob.iflags) < 1:
            mob.iflags = "0"
        if len(mob.rflags) < 1:
            mob.rflags = "0"
        if len(mob.vflags) < 1:
            mob.vflags = "0"
                
    def create_new(self):
        if len(self.list)>0:
            return Area.Mobile(Area.vnum_inc(self.list[len(self.list)-1].vnum))
        return Area.Mobile()
        
    def fillValues(self, mobile):
        self.vnum.set_text(mobile.vnum)
        self.name.set_text(mobile.name)
        self.short.set_text(mobile.short)
        self.long.set_text(mobile.long)

        self.look.delete_text(0,-1)
        self.look.insert_defaults(mobile.look)

        self.race.entry.set_text(mobile.race)
        self.align.set_text("%d" % mobile.align)
        self.mobgroup.set_text("%d" % mobile.mobgroup)
        self.level.set_text("%d" % mobile.level)
        self.toHit.set_text(mobile.toHit)
        self.hpDice.set_text(mobile.hpDice)
        self.dtype.set_text(mobile.dtype)
        self.damageDice.set_text(mobile.damageDice)
        self.pierce.set_text("%d" % mobile.pierce)
        self.bash.set_text("%d" % mobile.bash)
        self.slash.set_text("%d" % mobile.slash)
        self.magic.set_text("%d" % mobile.magic)
        self.start.set_text(mobile.start)
        self.default.set_text(mobile.default)
        self.sex.set_text(mobile.sex)
        self.treasure.set_text("%d" % mobile.treasure)

#        self.size.set_text(mobile.size)
        self.death.set_text(mobile.death)

        setFlags(mobile.actflags, self.actflags)
        setFlags(mobile.offflags, self.offflags)
        setFlags(mobile.affflags, self.affflags)
        setFlags(mobile.partflags, self.partflags)
        setFlags(mobile.formflags, self.formflags)

        for f in self.irvflags:
            (flag, i, r, v) = f
            if (string.find(mobile.iflags, flag)>=0):
                i.set_active(TRUE)
            else:
                i.set_active(FALSE)
            if (string.find(mobile.rflags, flag)>=0):
                r.set_active(TRUE)
            else:
                r.set_active(FALSE)
            if (string.find(mobile.vflags, flag)>=0):
                v.set_active(TRUE)
            else:
                v.set_active(FALSE)
    def empty(self):
        self.vnum.set_text("")
        self.name.set_text("")
        self.short.set_text("")
        self.long.set_text("")
        self.look.delete_text(0,-1)

        self.race.entry.set_text("")
        self.align.set_text("")
        self.mobgroup.set_text("")
        self.level.set_text("")
        self.toHit.set_text("")
        self.hpDice.set_text("")
        self.dtype.set_text("")
        self.damageDice.set_text("")
        self.pierce.set_text("")
        self.bash.set_text("")
        self.slash.set_text("")
        self.magic.set_text("")
        self.start.set_text("")
        self.default.set_text("")
        self.sex.set_text("")
        self.treasure.set_text("")
        self.death.set_text("")

        setFlags("", self.actflags)
        setFlags("", self.offflags)
        setFlags("", self.affflags)
        setFlags("", self.partflags)
        setFlags("", self.formflags)

        for f in self.irvflags:
            (flag, i, r, v) = f
            i.set_active(FALSE)
            r.set_active(FALSE)
            v.set_active(FALSE)
