#  AreaEd - an editor for rom-style areas, especially for
#  Dawn of the Ages  http://dawn.netcologne.de
#
#  Copyright (C) 2001 Peter Herth <p.herth@uni-koeln.de>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


from gtk import *
from gnome.ui import *
from GtkExtra import *
from GDK import *
from GTK import *


from StringIO import *
import cPickle
import Area

MOBDESCHEIGHT = 80
ITEMDESCHEIGHT = 80
ROOMDESCHEIGHT = 80



XPAD = 1
YPAD = 1

dnd_targets = [('pickle/mob', 0, -1),
               ('pickle/item', 0, -1)]

targets_mob = [('pickle/mob', 0, -1)]
targets_item = [('pickle/item', 0, -1)]


locationfactor ={"D" : 0.25,   #torso   
                 "J":  0.18,   #shield
                 "F":  0.13,   #leggings
                 "E":  0.10,   #helmet
                 "I":  0.10,   #armguards
                 "H":  0.06,   #hands
                 "G":  0.06,   #feet
                 "L":  0.03,   #waist
                 "K":  0.03,   #aboutbody
                 "M":  0.02,   #wrist
                 "C":  0.02}   #neck
                # "other pieces of "armour"       2}   #other
##                      <10           10 - 20         20  - 30        30 - 40            40 - 50          50 - 60

##Non Limited           15(0)          17(0)            19(0)          21(0)              23(0)            25(0)
##Limited (2)           20(0)          22(0)            24(0)          26(0)              28(0)            30(0)
##Limited (1)           26(0)          28(0)            30(0)          32(0)              34(0)            36(0)
##Legendary             34(0)          36(0)            38(0)          40(0)              42(0)            45(0)

def totAc(level,limit):
    if limit < 0:
        return 140+(level/50.0)*110
    elif limit == 0:
        return 330+(level/50.0)*110
    elif limit == 1:
        return 250+(level/50.0)*110
    else:
        return 190+(level/50.0)*110


        

def calcAc(item,limit):
    level = item.level
    total = totAc(level,limit)
    wear = item.wearflags
    for k in locationfactor.keys():
        if (string.find(wear, k)>=0):
            return round(total * locationfactor[k])
    return -1


def op_dir(dir):
    if dir == 0:
        return 2
    elif dir == 1:
        return 3
    elif dir == 2:
        return 0
    elif dir == 3:
        return 1
    elif dir == 4:
        return 5
    else:
        return 4

def mklabel(name,table,x,y):
    label = GtkLabel(name)
    label.show()
    table.attach(label,x,x+1,y,y+1,SHRINK,0, XPAD, YPAD)
    return label

def mkentry(table, x, y, x2=-1):
    entry = GtkEntry()
    entry.set_usize(50,24)
    entry.show()
    if (x2==-1) :
        x2 = x+1
    table.attach(entry,x,x2,y,y+1,SHRINK|EXPAND|FILL,0, XPAD, YPAD)
    return entry

def mk_le(name,table,x,y,x2=-1):
    label = GtkLabel(name)
    label.set_justify(JUSTIFY_LEFT)
    label.show()
    table.attach(label,x,x+1,y,y+1,0,0, XPAD, YPAD)
    entry = mkentry(table, x+1, y, x2)
    return entry

def mk_cb(name,table,x,y,x2=-1):
    label = GtkLabel(name)
    label.show()
    table.attach(label,x,x+1,y,y+1,0,0, XPAD, YPAD)
    entry = GtkCombo()
    entry.set_usize(50,24)
    table.attach(entry,x+1, x+2,y,y+1, SHRINK|EXPAND|FILL, 0, XPAD, YPAD)

    return entry

def mk_bu(name,table,x,y,x2=-1, xflags=0, yflags=0):
    button = GtkButton(name)
    if (x2==-1) :
        x2 = x+1
    button.show()
    table.attach(button,x,x2,y,y+1,xflags,yflags, XPAD, YPAD)
    return button

def mk_gbu(typ,table,x,y,x2=-1):
    button = GnomeStockButton(typ)
    if (x2==-1) :
        x2 = x+1
    button.show()
    table.attach(button,x,x2,y,y+1,SHRINK|EXPAND|FILL,0, XPAD, YPAD)
    return button

def mk_chk(name,table, x,y, xflags=FILL, yflags=0):
    box = GtkCheckButton(name)
    table.attach(box,x,x+1,y,y+1,xflags,yflags)
    return box

def createFlags(name,list,tooltips):
    file = open(name, "r")
    table = GtkTable(4,8)
    x = 0
    y = 0
    for line in file.readlines():
        l = string.split(line,"|")
        if len(l)>1:
            chk = mk_chk(l[1],table, x, y)
            tooltips.set_tip(chk, l[2])
            list.append( (l[0], chk))
            x = x +1
            if (x > 3):
                x = 0
                y = y+1
                    
    file.close()
    return table



def setFlags(mobileflags, flags):
    """set all flags contained int mobileflags in flags,
    flags is a list containing pairs of a flag and a checkbutton
    linked to that flag"""
    for f in flags:
        (flag, button) = f
        if (string.find(mobileflags, flag)>=0):
            button.set_active(TRUE)
        else:
            button.set_active(FALSE)


def getFlags(flags): 
    """return list of flags set in the buttonlist flags as a string"""
    mobflags = ""
    for f in flags:
        (flag, button) = f
        if (button.get_active()):
            mobflags = mobflags + flag
    if (len(mobflags)<1):
        mobflags = "0"
    return mobflags
           
           

class ExtraEdit(GtkTable):
    extra=[]
    actextra = 0
    ext = None
    
    def __init__(self, list):
        self.extra = list
        GtkTable.__init__(self,2,4)
        box = GtkHBox()

        add = GnomeStockButton(STOCK_PIXMAP_ADD)
        rem = GnomeStockButton(STOCK_PIXMAP_REMOVE)
        prev = GnomeStockButton(STOCK_BUTTON_PREV)
        next = GnomeStockButton(STOCK_BUTTON_NEXT)
        add.connect("clicked", self.new)
        rem.connect("clicked", self.remv)
        prev.connect("clicked", self.prev)
        next.connect("clicked", self.next)
        box.pack_start(add,0,0)
        box.pack_start(rem,0,0)
        box.pack_start(prev,0,0)
        box.pack_start(next,0,0)
        self.attach(box, 1,2,0,1,FILL|EXPAND,0)
        self.name = mk_le("keywords:",self, 0,1)
        mklabel("text:",self, 0,2)
        
        text = GtkText()
        text.set_editable(TRUE)
        text.set_usize(1,64)
        text.show()
        self.attach(text,1,2,2,3,FILL|EXPAND|SHRINK,EXPAND|FILL, XPAD, YPAD)
        self.text=text

    def setExtraList(self, list):
        self.extra = list
        self.actextra = 0
        if len(list)>0:
            self.set(list[0])
        else:
            self.set(None)
            
    def getExtraList(self):
        self.save()
        return self.extra

    def save(self):
        if self.ext == None:
            return

        extra = self.ext
        extra.name = self.name.get_text()
        extra.text = self.text.get_chars(0, self.text.get_length())
 
    def set(self, extra):
        self.save()
        if extra:
            self.ext = extra
            self.name.set_text(extra.name)
            self.text.delete_text(0,-1)
            self.text.insert_defaults(extra.text)
        else:
            self.ext = None
            self.name.set_text("")
            self.text.delete_text(0,-1)

    def next(self, *arg):
        if self.actextra < (len(self.extra)-1):
            self.actextra = self.actextra +1
            self.set(self.extra[self.actextra])

    def prev(self, *arg):
        if self.actextra > 0 and len(self.extra)>0:
            self.actextra = self.actextra - 1
            self.set(self.extra[self.actextra])

    def new(self, *arg):
        name = ""
        if (len(self.extra)>0):
            name = self.extra[-1].name
        extra = Extra(vnum_inc(name),"")
        self.set(extra)
        self.extra.append(extra)
        self.actextra = len(self.extra)-1
        
    def remv(self, *arg):
        if self.ext == None:
            return
        GnomeOkCancelDialog(("Do you really want to delete the extra %s ?" % self.ext.name),
                            self.extra_rem, ed)

    def extra_rem(self, *args):
        erg = args[0]
        if not erg:
            if self.actextra > 0:
                self.actextra = self.actextra-1
            self.extra.remove(self.ext)
            if len(self.extra) > 0:
                self.set(self.extra[self.actextra])
            else:
                self.set(None)

    


class EditForm(GtkTable):
    list=[]
    actnr = 0
    actitem = None
    
    def __init__(self, ed, list):
        self.list = list
        GtkTable.__init__(self,2,4)
        box = GtkHBox()
        self.ed = ed
        add = GnomeStockButton(STOCK_PIXMAP_ADD)
        rem = GnomeStockButton(STOCK_PIXMAP_REMOVE)
        prev = GnomeStockButton(STOCK_BUTTON_PREV)
        next = GnomeStockButton(STOCK_BUTTON_NEXT)
        add.connect("clicked", self.new)
        rem.connect("clicked", self.remv)
        prev.connect("clicked", self.prev)
        next.connect("clicked", self.next)
        self.bu_add = add
        self.bu_rem = rem
        self.bu_prev= prev
        self.bu_next = next
        box.pack_start(add,0,0)
        box.pack_start(rem,0,0)
        box.pack_start(prev,0,0)
        box.pack_start(next,0,0)
        self.attach(box, 1,6,0,1,FILL|EXPAND,0)
        self.setButtonStates()

    def setValues(self, item):
        pass

    def fillValues(self, item):
        pass

    def empty(self):
        pass

    def create_new(self):
        pass

    def setButtonStates(self):
        if self.actitem:
            self.bu_rem.set_sensitive(1)
        else:
            self.bu_rem.set_sensitive(0)
            
        if len(self.list)>self.actnr+1:
            self.bu_next.set_sensitive(1)
        else:
            self.bu_next.set_sensitive(0)

        if self.actnr>0:
            self.bu_prev.set_sensitive(1)
        else:
            self.bu_prev.set_sensitive(0)

            

    def setList(self, list):
        self.list = list
        self.actnr = 0
        if len(list)>0:
            self.set(list[0])
        else:
            self.set(None)
        self.setButtonStates()
            
    def getList(self):
        self.save()
        return self.list

    def save(self):
        if self.actitem == None:
            return
        self.setValues(self.actitem)

 
    def set(self, item):
        self.save()
        if item:
            self.fillValues(item)
            self.actitem = item
        else:
            self.actitem = None
            self.empty()
            

    def next(self, *arg):
        if self.actnr < (len(self.list)-1):
            self.actnr = self.actnr +1
            self.set(self.list[self.actnr])
        self.setButtonStates()
        
    def prev(self, *arg):
        if self.actnr> 0 and len(self.list)>0:
            self.actnr = self.actnr - 1
            self.set(self.list[self.actnr])
        self.setButtonStates()

    def new(self, *arg):
        new_item = self.create_new()
##        self.set(new_item)
        self.list.append(new_item)
        self.actnr = len(self.list)-1
        self.setButtonStates()
        self.ed.refresh_lists()
        self.set(new_item)
##        name = ""
##        if (len(self.extra)>0):
##            name = self.extra[-1].name
##        extra = Extra(vnum_inc(name),"")
##        self.set(extra)
##        self.extra.append(extra)
##        self.actextra = len(self.extra)-1
        
    def remv(self, *arg):
        if self.actitem == None:
            return
        GnomeOkCancelDialog(("Do you really want to delete the mobile %s ?" % self.actitem.name),
                            self.item_rem, self.ed)

    def item_rem(self, *args):
        erg = args[0]
        if not erg:
            if self.actnr > 0:
                self.actnr = self.actnr-1
            if self.actitem:
                self.list.remove(self.actitem)
            if len(self.list) > 0:
                self.set(self.list[self.actnr])
            else:
                self.set(None)
        self.setButtonStates()


def scrolled(widget):
    swin = GtkScrolledWindow()
    swin.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
    swin.add_with_viewport(widget)
    return swin

class ListForm(GtkWindow):
    selmob = None
    selitem = None
    selroom = None
    
    def __init__(self, ed):
        GtkWindow.__init__(self, title="AreaEd")
        self.area = ed.area
        self.ed = ed
        
        notebook = GtkNotebook()
        notebook.set_tab_pos = POS_TOP

        self.mobs = GtkList()
        self.mobs.set_selection_mode(SELECTION_BROWSE)
        self.items = GtkList()
        self.items.set_selection_mode(SELECTION_BROWSE)
        self.rooms = GtkList()
        self.rooms.set_selection_mode(SELECTION_BROWSE)

        self.mobs.connect("select_child", self.mobclick)
        self.items.connect("select_child", self.itemclick)
        self.rooms.connect("select_child", self.roomclick)

        notebook.append_page(scrolled(self.mobs), GtkLabel("Mobs"))
        notebook.append_page(scrolled(self.items), GtkLabel("Items"))
        notebook.append_page(scrolled(self.rooms), GtkLabel("Rooms"))
        self.add(notebook)
        self.connect("delete_event", self.eclose)
        self.set_default_size(200,300)


        #self.mobs.connect("drag_begin", self.drag_start)
        self.mobs.connect('drag_data_get', self.mob_dnd_drag_data_get)
        self.mobs.drag_source_set(GDK.BUTTON1_MASK, [('pickle/mob', 0, -1)], GDK.ACTION_COPY)
        self.items.connect('drag_data_get', self.item_dnd_drag_data_get)
        self.items.drag_source_set(GDK.BUTTON1_MASK, [('pickle/mob', 0, -1)], GDK.ACTION_COPY)
        self.rooms.connect('drag_data_get', self.room_dnd_drag_data_get)
        self.rooms.drag_source_set(GDK.BUTTON1_MASK, [('pickle/room', 0, -1)], GDK.ACTION_COPY)


        self.show_all()

    def mob_dnd_drag_data_get(self, w, context, selection_data, info, time):
        sio = StringIO()    
        cPickle.dump(self.selmob, sio)
        dnd_string = sio.getvalue()
        selection_data.set(selection_data.target, 8,dnd_string)
    def item_dnd_drag_data_get(self, w, context, selection_data, info, time):
        sio = StringIO()    
        cPickle.dump(self.selitem, sio)
        dnd_string = sio.getvalue()
        selection_data.set(selection_data.target, 8,dnd_string)
    def room_dnd_drag_data_get(self, w, context, selection_data, info, time):
        sio = StringIO()    
        cPickle.dump(self.selroom, sio)
        dnd_string = sio.getvalue()
        selection_data.set(selection_data.target, 8,dnd_string)

    def eclose(self, *arg):
        self.hide_all()
        return TRUE

    def set_area(self, area):
        self.area = area
        self.mobs.clear_items(0,-1)
        self.items.clear_items(0,-1)
        self.rooms.clear_items(0,-1)

        mlist = self.moblist(area.mobs)
        self.mobs.append_items(mlist)
        ilist = self.itemlist(area.items)
        self.items.append_items(ilist)
        rlist = self.roomlist(area.rooms)
        self.rooms.append_items(rlist)
##        self.show_all()


    def moblist(self, mobs):
        erg = []
        for m in mobs:
            i = GtkListItem("[%s] %s" % (m.vnum, m.short))
            i.set_data("mob", m)
            i.show()
            erg.append(i)
        return erg

    def itemlist(self, items):
        erg = []
        for m in items:
            i = GtkListItem("[%s] %s" % (m.vnum, m.short))
            i.set_data("item", m)
            i.show()
            erg.append(i)
        return erg
    def roomlist(self, rooms):
        erg = []
        for m in rooms:
            i = GtkListItem("[%s] %s" % (m.vnum, m.name))
            i.set_data("room", m)
            i.show()
            erg.append(i)
        return erg

    def mobclick(self, *args):
        mob = args[1].get_data("mob")
        self.selmob = mob
        self.ed.setmob(mob)
    def itemclick(self, *args):
        item = args[1].get_data("item")
        self.selitem = item
        self.ed.setitem(item)
    def roomclick(self, *args):
        room = args[1].get_data("room")
        self.selroom = room
        self.ed.setroom(room)
